<?php

namespace Drupal\openid_claveunica\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Url;
use Drupal\openid_claveunica\AccountProfileCheckerInterface;
use Drupal\user\AccountForm;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Session\SessionInterface;

/**
 * Form handler for the complete profile claveunica user form.
 */
class CompleteProfileClaveunicaUserForm extends AccountForm {

  /**
   * The account profile checker service.
   *
   * @var \Drupal\openid_claveunica\AccountProfileCheckerInterface
   */
  protected AccountProfileCheckerInterface $accountProfileChecker;

  /**
   * The session object.
   *
   * @var \Symfony\Component\HttpFoundation\Session\SessionInterface
   */
  protected SessionInterface $session;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->accountProfileChecker = $container->get('openid_claveunica.account_profile_checker');
    $instance->session = $container->get('session');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    if ($helptext = $form_state->get('client_settings')['help_complete_profile'] ?? FALSE) {
      $form['helptext'] = [
        '#markup' => $helptext,
        '#weight' => -999,
      ];
    }

    foreach (Element::children($form['account']) as $key) {
      if ($key == 'mail') {
        continue;
      }
      $form['account'][$key]['#access'] = FALSE;
    }
    $form['account']['mail']['#default_value'] = '';
    $form['account']['mail']['#access'] = TRUE;

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    /** @var \Drupal\user\UserInterface $account */
    $account = $this->entity;
    if (!$this->accountProfileChecker->checkHashCompleteProfile($account, $form_state->get('complete_profile_hash'))) {
      $this->messenger->addError($this->t('User ID cannot be identified.'));
      $response = new RedirectResponse(Url::fromRoute('<front>')->toString());
      $response->send();
    }
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\user\UserInterface $account */
    $account = $this->entity;
    $account->save();
    $this->logger('openid_claveunica')->info('%name filled missing fields.', ['%name' => $account->getAccountName()]);

    user_login_finalize($account);
    $this->messenger()->addStatus($this->t('Registration successful. You are now logged in.'));
    $form_state->setRedirect('<front>');
  }

  /**
   * {@inheritdoc}
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    $actions['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Complete'),
      '#submit' => ['::submitForm', '::save'],
    ];

    return $actions;
  }

}
